
#include "datatypes.h"
#include "systime.h"
#include "lpc21xx.h"
#include "interrupts.h"
#include "can.h"
#include "can_user.h"


void  SYSTIME_ISR ( void) IRQ_FUNC_PROTO;




// IRQ function. First call after 10 ms, then every 16 ms.
void  SYSTIME_ISR ( void)
{
	CANMsg_t  TxMsg;
	
	
	// hardware irq acknowledge
	T1IR = 1 << 0;
	
	// set match register to next event (16 millis)
	T1MR0 += 16000;
	
	//************************************************
	// user code
	
	TxMsg.Id = 0x512;
	TxMsg.Len = 2;
	TxMsg.Type = CAN_MSG_EXTENDED;
	
	TxMsg.Data16[0] = 0x4512;
	
	CAN_UserWrite ( CAN_BUS1, &TxMsg);
	
	// end user code
	//************************************************
	
	// update priority hardware
	VICVectAddr = 0;
}





// init timer 1 as systemtimer
void  SYSTIME_Init ( void)
{

	// Timer config, 1 s Resolution

	// Timer halt
	T1TCR = 1 << 1;

	// Set Prescaler
	T1PR = 59;

	// first Match after 10 millis
	T1MCR = 1 << 0;		// interrupt on match, no stop, no reset
	T1MR0 = 10000;
	
	// config interrupt
	VICVectAddr5 = (u32_t) &SYSTIME_ISR;
	VICVectCntl5 = 1 << 5 | 5;
	
	//no Capture
	T1CCR = 0;

	// no external Toggles
	T1EMR = 0;

	// Clear all interrupts
	T1IR = 0xFF;
	
	// enable timer IRQ
	VICIntEnable = 1 << 5;

	// Timer start
	T1TCR = 1;

}

